//////////////////////////////////////////////////////////////////////
// Copyright 2003.    Moe Wheatley AE4JY  <http://www.moetronix.com>
//
//This program is free software; you can redistribute it and/or
//modify it how ever you wish.  Its primary goal is to provide an
// example for 3rd party software developers.  Support will be limited
// depending on my available time.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SDR14X.h"
#include "SDR14XCtl.h"
#include "SDR14XPpg.h"
#include "6620params.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//sample rate that above which the USB cannot keep up and so the data
// cannot be contiguous.
#define MAX_CONTIGUOUS_FREQ 197000

#define MAX_AUX_BYTES 24

#define DATA_BLOCK_SIZE 4096	//block size in 16 bit words of the SDR-14 data.

#define BLOCK_FIFO_SIZE 32	//must be power of 2
//#define BLOCK_FIFO_SIZE 8	//must be power of 2

#define CAPTURE_MODE_CONTIGUOUS 0	//Three capture modes of the SDR-14
#define CAPTURE_MODE_CONTINUOUS 1
#define CAPTURE_MODE_ONESHOT 2

#define DATA_ACK_TIMEOUT_VALUE ((66666666*1)/2048)	//1 seconds of 2048 sample blocks

#define IQ_OFFSET_CONST 307.0	//Used for NCO spur removal

#define MAX_WINMSG_LIMIT 1

#define IOEEROM_SAMPLERATE 0	//External IO EEROM memory address definitions for sdriq

#define NOMINAL_ADFREQ 66666667

IMPLEMENT_DYNCREATE(CSDR14XCtrl, COleControl)

/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CSDR14XCtrl, COleControl)
	//{{AFX_MSG_MAP(CSDR14XCtrl)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
	ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
	ON_MESSAGE( MSG_DATARDY, OnDataRdy)
	ON_MESSAGE( MSG_AUXDATARDY, OnAuxDataRdy)
	ON_MESSAGE( MSG_STATUSCHNG, OnStatusChange)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CSDR14XCtrl, COleControl)
	//{{AFX_DISPATCH_MAP(CSDR14XCtrl)
	DISP_PROPERTY_EX(CSDR14XCtrl, "ClockFrequency", GetClockFrequency, SetClockFrequency, VT_I4)
	DISP_PROPERTY_EX(CSDR14XCtrl, "DeviceName", GetDeviceName, SetDeviceName, VT_BSTR)
	DISP_PROPERTY_EX(CSDR14XCtrl, "FirmwareRevision", GetFirmwareRevision, SetFirmwareRevision, VT_I2)
	DISP_PROPERTY_EX(CSDR14XCtrl, "HFgain", GetHFgain, SetHFgain, VT_I2)
	DISP_PROPERTY_EX(CSDR14XCtrl, "IFfilter", GetIFfilter, SetIFfilter, VT_I2)
	DISP_PROPERTY_EX(CSDR14XCtrl, "InputSource", GetInputSource, SetInputSource, VT_I2)
	DISP_PROPERTY_EX(CSDR14XCtrl, "InterfaceVersion", GetInterfaceVersion, SetInterfaceVersion, VT_I2)
	DISP_PROPERTY_EX(CSDR14XCtrl, "NCOfrequency", GetNCOfrequency, SetNCOfrequency, VT_I4)
	DISP_PROPERTY_EX(CSDR14XCtrl, "SerialNumber", GetSerialNumber, SetSerialNumber, VT_BSTR)
	DISP_PROPERTY_EX(CSDR14XCtrl, "IFgain", GetIFgain, SetIFgain, VT_I2)
	DISP_PROPERTY_EX(CSDR14XCtrl, "SampleRate", GetSampleRate, SetSampleRate, VT_I4)
	DISP_PROPERTY_EX(CSDR14XCtrl, "GetBootRevision", GetBootRevision, SetBootRevision, VT_I2)
	DISP_PROPERTY_EX(CSDR14XCtrl, "IQOffset", GetIQOffset, SetIQOffset, VT_R8)
	DISP_FUNCTION(CSDR14XCtrl, "Stop", Stop, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CSDR14XCtrl, "CloseUartPort", CloseUartPort, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CSDR14XCtrl, "CloseSpiPort", CloseSpiPort, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CSDR14XCtrl, "CloseI2CPort", CloseI2CPort, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CSDR14XCtrl, "Start", Start, VT_EMPTY, VTS_I2 VTS_I2)
	DISP_FUNCTION(CSDR14XCtrl, "GetLongData", GetLongData, VT_I4, VTS_PI4)
	DISP_FUNCTION(CSDR14XCtrl, "GetDoubleData", GetDoubleData, VT_I4, VTS_PR8)
	DISP_FUNCTION(CSDR14XCtrl, "GetIntegerData", GetIntegerData, VT_I4, VTS_PI2)
	DISP_FUNCTION(CSDR14XCtrl, "GetUartData", GetUartData, VT_I2, VTS_PI2)
	DISP_FUNCTION(CSDR14XCtrl, "OpenI2CPort", OpenI2CPort, VT_EMPTY, VTS_I2)
	DISP_FUNCTION(CSDR14XCtrl, "OpenSpiPort", OpenSpiPort, VT_EMPTY, VTS_I2)
	DISP_FUNCTION(CSDR14XCtrl, "OpenUartPort", OpenUartPort, VT_EMPTY, VTS_I4 VTS_I2 VTS_BOOL)
	DISP_FUNCTION(CSDR14XCtrl, "SendUartData", SendUartData, VT_EMPTY, VTS_PI2 VTS_I2)
	DISP_FUNCTION(CSDR14XCtrl, "Send6620RawCmd", Send6620RawCmd, VT_EMPTY, VTS_I2 VTS_I2 VTS_I2 VTS_I2 VTS_I2 VTS_I2)
	DISP_FUNCTION(CSDR14XCtrl, "SendI2CData", SendI2CData, VT_EMPTY, VTS_PI2 VTS_I2)
	DISP_FUNCTION(CSDR14XCtrl, "SendSpiData", SendSpiData, VT_EMPTY, VTS_I4 VTS_I4 VTS_I2)
	DISP_FUNCTION(CSDR14XCtrl, "GetFftScreenData", GetFftScreenData, VT_BOOL, VTS_I4 VTS_I4 VTS_R8 VTS_R8 VTS_I4 VTS_I4 VTS_PI4)
	DISP_FUNCTION(CSDR14XCtrl, "GetFftDoubleData", GetFftDoubleData, VT_BOOL, VTS_PR8)
	DISP_FUNCTION(CSDR14XCtrl, "SetupFFT", SetupFFT, VT_EMPTY, VTS_I4 VTS_I2 VTS_BOOL)
	DISP_FUNCTION(CSDR14XCtrl, "ResetFftAverage", ResetFftAverage, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CSDR14XCtrl, "InitControl", InitControl, VT_EMPTY, VTS_NONE)
	//}}AFX_DISPATCH_MAP
	DISP_FUNCTION_ID(CSDR14XCtrl, "AboutBox", DISPID_ABOUTBOX, AboutBox, VT_EMPTY, VTS_NONE)
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CSDR14XCtrl, COleControl)
	//{{AFX_EVENT_MAP(CSDR14XCtrl)
	EVENT_CUSTOM("StatusChange", FireStatusChange, VTS_I4)
	EVENT_CUSTOM("DataRdy", FireDataRdy, VTS_I4)
	EVENT_CUSTOM("UartDataRdy", FireUartDataRdy, VTS_NONE)
	//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CSDR14XCtrl, 1)
	PROPPAGEID(CSDR14XPropPage::guid)
END_PROPPAGEIDS(CSDR14XCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CSDR14XCtrl, "Moe.SDR14XCtrl.1",
	0x445ec34f, 0xeca5, 0x4e44, 0xbc, 0x8f, 0xdd, 0x98, 0x5f, 0x79, 0x23, 0xc9)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CSDR14XCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DSDR14X =
		{ 0xb087b26e, 0x3998, 0x4f4e, { 0xb9, 0xa5, 0x39, 0x2b, 0x51, 0x92, 0x18, 0xff } };
const IID BASED_CODE IID_DSDR14XEvents =
		{ 0x457c702, 0xe9ab, 0x4a87, { 0xa6, 0x3c, 0xf0, 0x53, 0x30, 0xa1, 0xe7, 0xd9 } };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwSDR14XOleMisc =
#ifndef OLEMISC_INVISIBLEATRUNTIME_FIX
	//(dec) 9-Oct-04 Added to fix OLEMISC_INVISIBLEATRUNTIME problem with VB
	OLEMISC_INVISIBLEATRUNTIME |
#endif
	OLEMISC_ACTIVATEWHENVISIBLE |
	OLEMISC_SETCLIENTSITEFIRST |
	OLEMISC_INSIDEOUT |
	OLEMISC_CANTLINKINSIDE |
	OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CSDR14XCtrl, IDS_SDR14X, _dwSDR14XOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CSDR14XCtrl::CSDR14XCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CSDR14XCtrl

BOOL CSDR14XCtrl::CSDR14XCtrlFactory::UpdateRegistry(BOOL bRegister)
{
	// TODO: Verify that your control follows apartment-model threading rules.
	// Refer to MFC TechNote 64 for more information.
	// If your control does not conform to the apartment-model rules, then
	// you must modify the code below, changing the 6th parameter from
	// afxRegApartmentThreading to 0.

	if (bRegister)
		return AfxOleRegisterControlClass(
			AfxGetInstanceHandle(),
			m_clsid,
			m_lpszProgID,
			IDS_SDR14X,
			IDB_SDR14X,
			afxRegApartmentThreading,
			_dwSDR14XOleMisc,
			_tlid,
			_wVerMajor,
			_wVerMinor);
	else
		return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CSDR14XCtrl::CSDR14XCtrl - Constructor

CSDR14XCtrl::CSDR14XCtrl()
{
	InitializeIIDs(&IID_DSDR14X, &IID_DSDR14XEvents);
#ifdef OLEMISC_INVISIBLEATRUNTIME_FIX
	//(dec) 9-Oct-04 Added to fix OLEMISC_INVISIBLEATRUNTIME problem with VB
	m_bDrawCtl = TRUE;
#endif
	m_ThreadRunning = FALSE;
	m_pUSBThread = NULL;
	m_pFIFOThread = NULL;
	m_pDataBuffer = NULL;
	m_DeviceName = _T("Unknown");
	m_SerialNumber= _T("??????");
	m_BootRevision = 0;
	m_FirmwareRevision = 0;
	m_HFgain = 0;
	m_IFgain = 24;
	m_IFfilter = 0;
	m_InputSource = 0;
	m_InterfaceVersion = 0;
	m_NCOfrequency = 15000000;
	m_ClockFrequency = 66666667;
	m_CaptureMode = CAPTURE_MODE_CONTIGUOUS;
	m_SampleRate = m_ClockFrequency;
	m_NumberOfBlocks = -1;
	m_NumberOfSPIBits = 0;
	m_Complex = FALSE;
	m_FFTsize = 0;
	m_FFTInvert = FALSE;
	m_FFTAve = 1;
	m_IQOffset = 0.0;
	m_FifoHead = 0;
	m_FifoTail = 0;
	m_FifoFFTTail = 0;
	m_AuxRxHead = 0;
	m_AuxRxTail = 0;
	m_WindowsMsgCount = 0;
	m_TestModeChannel = 0;
	InitializeCriticalSection(&m_CriticalSection);
}


/////////////////////////////////////////////////////////////////////////////
// CSDR14XCtrl::~CSDR14XCtrl - Destructor

CSDR14XCtrl::~CSDR14XCtrl()
{
	EndThreads();
	if(m_pUSBThread)
		delete m_pUSBThread;
	if(m_pFIFOThread)
		delete m_pFIFOThread;
	if(m_pDataBuffer)
	{
		for(int i=0; i<BLOCK_FIFO_SIZE; i++)
		{
			if(m_pDataBuffer[i])
			{
				delete[] m_pDataBuffer[i];
				m_pDataBuffer[i] = NULL;
			}
		}
		delete[] m_pDataBuffer;
		m_pDataBuffer = NULL;
	}
	DeleteCriticalSection(&m_CriticalSection);
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//  S y s t e m / C o n t r o l   F u n c t i o n s 
//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

/////////////////////////////////////////////////////////////////////////////
// CSDR14XCtrl::OnDraw - Drawing function (not used)
/////////////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::OnDraw(
			CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
	if ( m_hWnd == NULL )
		return;

#ifdef OLEMISC_INVISIBLEATRUNTIME_FIX
	//(dec) 9-Oct-04 Added to fix OLEMISC_INVISIBLEATRUNTIME problem with VB
	if ( !m_bDrawCtl )
		return;
#endif

	//(dec) 4-Oct-04 This is for designer-mode rendering
	if ( theApp.m_hAppIcon != NULL )
	{
		if ( rcBounds.Width() != 32 || rcBounds.Height() != 32 )
			SetControlSize(32, 32);
		pdc->DrawIcon(0, 0, theApp.m_hAppIcon);
	}
	else
	{
		pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(WHITE_BRUSH)));
		pdc->Ellipse(rcBounds);
	}
}


/////////////////////////////////////////////////////////////////////////////
// CSDR14XCtrl::DoPropExchange - Persistence support

void CSDR14XCtrl::DoPropExchange(CPropExchange* pPX)
{
	ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
	COleControl::DoPropExchange(pPX);

	// TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CSDR14XCtrl::OnResetState - Reset control to default state

void CSDR14XCtrl::OnResetState()
{
 	COleControl::OnResetState();  // Resets defaults found in DoPropExchange
}


/////////////////////////////////////////////////////////////////////////////
// CSDR14XCtrl::AboutBox - Display an "About" box to the user

void CSDR14XCtrl::AboutBox()
{
	CDialog dlgAbout(IDD_ABOUTBOX_SDR14X);
	dlgAbout.DoModal();
}


//////////////////////////////////////////////////////////////////////
//  Called by system when control is closing
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::OnClose() 
{
	EndThreads();	
	CWnd::OnClose();
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//   C X S D R 1 4 C t r l    E v e n t     H a n d l e r s
//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//////////////////////////////////////////////////////////////////////
// Process User message MSG_DATARDY from thread by firing ActiveX event
//////////////////////////////////////////////////////////////////////
afx_msg LRESULT CSDR14XCtrl::OnDataRdy(UINT BlocksAvailable, LONG p2)
{
	FireDataRdy(BlocksAvailable);
	return 0;
}

//////////////////////////////////////////////////////////////////////
// Process User message MSG_AUXDATARDY from thread by firing ActiveX event
//////////////////////////////////////////////////////////////////////
afx_msg LRESULT CSDR14XCtrl::OnAuxDataRdy(UINT length, LONG p2)
{
	FireUartDataRdy();
	return 0;
}

//////////////////////////////////////////////////////////////////////
// Process User message MSG_STATUSCHNG from thread by firing ActiveX event
//////////////////////////////////////////////////////////////////////
afx_msg LRESULT CSDR14XCtrl::OnStatusChange(UINT status, LONG p2)
{
	FireStatusChange(status);
	return 0;
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//   L o c a l    H e l p e r    F u n c t i o n s
//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//////////////////////////////////////////////////////////////////////
// Worker Thread static procedure that just calls the main USB process loop
//////////////////////////////////////////////////////////////////////
UINT CSDR14XCtrl::USBThreadProc(LPVOID pParam)
{
CSDR14XCtrl* pSDR14Ctrl = (CSDR14XCtrl*)pParam;
	Sleep(500);		//don't start right away
	pSDR14Ctrl->m_USBio.USBthreadProc( pSDR14Ctrl);
	return 2;
}

//////////////////////////////////////////////////////////////////////
// Worker Thread static procedure that just calls the main FIFO process loop
//////////////////////////////////////////////////////////////////////
UINT CSDR14XCtrl::FIFOThreadProc(LPVOID pParam)
{
CSDR14XCtrl* pSDR14Ctrl = (CSDR14XCtrl*)pParam;
	Sleep(300);		//don't start right away
	pSDR14Ctrl->m_USBio.FIFOThreadProc( pSDR14Ctrl);
	return 3;
}

//////////////////////////////////////////////////////////////////////
//  Ends thread by telling it to stop and waiting on it to quit
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::EndThreads()
{
	if(m_pUSBThread)
	{
		m_ThreadRunning = FALSE;
		m_pUSBThread->ResumeThread();
		::WaitForSingleObject(m_pUSBThread->m_hThread, INFINITE);
		delete m_pUSBThread;
		m_pUSBThread = NULL;
	}
	if(m_pFIFOThread)
	{
		m_ThreadRunning = FALSE;
		m_pFIFOThread->ResumeThread();
		::WaitForSingleObject(m_pFIFOThread->m_hThread, INFINITE);
		delete m_pFIFOThread;
		m_pFIFOThread = NULL;
	}
}

///////////////////////////////////////////////////////////////////////////
//  Parses received ASCP format message from SDR-14
//  Called by worker thread in CUSBio object.
///////////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::ParseASCPmsg(ASCPstruct* pASCPmsg)
{
int i;
int j;
	switch( pASCPmsg->bytes.b1 & TYPE_MASK)	//ASCP message type
	{
		case TYPE_TARG_RESP_CITEM:
			switch( pASCPmsg->words.CItem )
			{
				case CI_GENERAL_INTERFACE_NAME:
					m_DeviceName = (char*)pASCPmsg->bytes.parms;
					ReadSDRIQClockFrequency();	//get calibrated reference frequency from USB chip EEROM
					WriteSDRIQClockFrequency();	//echo it back to the SDRIQ processor
					break;
				case CI_GENERAL_INTERFACE_SERIALNUM:
					m_SerialNumber = (char*)pASCPmsg->bytes.parms;
					break;
				case CI_GENERAL_HARDFIRM_VERSION:
					if(1 == pASCPmsg->bytes.parms[0])
						m_FirmwareRevision = (unsigned __int16)pASCPmsg->bytes.parms[1];
					else
						m_BootRevision = (unsigned __int16)pASCPmsg->bytes.parms[1];
					break;
				case CI_GENERAL_INTERFACE_VERSION:
					m_InterfaceVersion = pASCPmsg->words.parms[0];
					//signal status that the SDR-14 is ready for use
					::PostMessage( m_hWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
					break;
				case CI_RX_STATE:
					if(pASCPmsg->bytes.parms[1] == RX_STATE_ON)
					{
						m_USBio.m_Running = TRUE;
						::PostMessage( m_hWnd, MSG_STATUSCHNG, STATUS_RUNNING, 0);
					}
					else
					{
						m_USBio.m_Running = FALSE;
						::PostMessage( m_hWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
					}
					break;
				case CI_RX_IF_GAIN:
					i=0;
					break;
				default:
//TRACE("y");
					break;
			}
			break;
		case TYPE_TARG_UNSOLICITED_CITEM:
			switch( pASCPmsg->words.CItem )
			{
				case CI_RX_STATE:	//unsolicited on/offs used for data syncing
					if(pASCPmsg->bytes.parms[1] == RX_STATE_ON)
					{
						m_BlockCount = 0;
					}
					else if(pASCPmsg->bytes.parms[1] == RX_STATE_IDLE)
					{
						m_BlockCount = 0;
						m_USBio.m_Running = FALSE;
						::PostMessage( m_hWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
					}
					else
					{
						m_BlockCount = 0;
						m_USBio.m_Running = FALSE;
						::PostMessage( m_hWnd, MSG_STATUSCHNG, STATUS_ERROR, 0);
					}
					break;
				case CI_GENERAL_STATUS_CODE:	//unsolicited status msgs from sdr
					switch( pASCPmsg->bytes.parms[0] )
					{
						case GENERAL_STATUS_IDLE:
							if(!m_USBio.m_Running)
								::PostMessage( m_hWnd, MSG_STATUSCHNG, STATUS_IDLECONNECTED, 0);
							break;
						case GENERAL_STATUS_BUSY:
							if(m_USBio.m_Running)
								::PostMessage( m_hWnd, MSG_STATUSCHNG, STATUS_RUNNING, 0);
							break;
						case GENERAL_STATUS_6620LOADING:
							break;
						case GENERAL_STATUS_BOOTIDLE:
							break;
						case GENERAL_STATUS_BOOTBUSY:
							break;
						case GENERAL_STATUS_BOOTERROR:
							break;
						case GENERAL_STATUS_ADOVERLOAD:
							if(!m_ADOverload)
							{
//TRACE("\n\rOvr");
								::PostMessage( m_hWnd, MSG_STATUSCHNG, STATUS_ADOVERLOAD, 0);
								m_ADOverload = TRUE;
							}
							break;
					}
					break;
				default:
TRACE("z");
					break;
			}
			break;
		case TYPE_TARG_RESP_CITEM_RANGE:
			break;
		case TYPE_DATA_ITEM_ACK:	//acks for all the data items that ack
			switch( pASCPmsg->words.CItem )
			{
				case 0:	//acks for data0 blocks (firmware update blocks)
					break;
				case 1: //acks for data1 blocks (6620 parameter loads)
					break;
				case 2: 
					break;
				case 3:
					break;
			}
			break;
		case TYPE_TARG_DATA_ITEM0:	//main data item from SDR-14
			EnterCriticalSection(&m_CriticalSection);
			//put data block into DataBuffer FIFO
			for(i=0, j=m_BlockCount*DATA_BLOCK_SIZE; i<DATA_BLOCK_SIZE; i++,j++)
				m_pDataBuffer[m_FifoHead][j] = (SHORT)pASCPmsg->data.parms[i];
			m_BlockCount++;
			if(m_BlockCount >= m_NumberOfBlocks)
			{
//				if(m_WindowsMsgCount < MAX_WINMSG_LIMIT)
				{
					m_FifoHead++;
					m_FifoHead &= (BLOCK_FIFO_SIZE-1);
					::PostMessage( m_hWnd, MSG_DATARDY, m_BlockCount, 0);
//					m_WindowsMsgCount++;
				}
//				else
//				{
//					m_BlockCount = 0;
//TRACE(".");
//				}
				m_BlockCount = 0;
			}
			LeaveCriticalSection(&m_CriticalSection);
			// send a Data0 ack every second or so to keep the SDR-14
			//  watchdog from timing out
			if(--m_DataAckCounter <= 0)
			{
				m_DataAckCounter = m_DataAckCounterLoadVal;
				m_ASCPackmsg.words.header = DATA_ITEM_ACK_LENGTH;
				m_ASCPackmsg.bytes.b1 |= TYPE_DATA_ITEM_ACK;
				m_ASCPackmsg.bytes.b2 = 0;
				m_USBio.SendASCPMsg( &m_ASCPackmsg);
			}
			break;
		case TYPE_TARG_DATA_ITEM1:
			break;
		case TYPE_TARG_DATA_ITEM2:	//UART rx data message from SDR-14
			for(i=0; i<(pASCPmsg->words.header&LENGTH_MASK - 2); i++)
			{
				EnterCriticalSection(&m_CriticalSection);
				m_AuxRxQue[m_AuxRxHead++] = pASCPmsg->buf[i+2];
				m_AuxRxHead &= AUX_QUE_MODMASK;	//deal with queue ptr wrap around
				LeaveCriticalSection(&m_CriticalSection);
			}
//TRACE("\n\rl=%d h=%d t=%d",i,m_AuxRxHead, m_AuxRxTail);
			::PostMessage( m_hWnd, MSG_AUXDATARDY, i, 0);
			break;
		case TYPE_TARG_DATA_ITEM3:
			break;
		default:
TRACE("x");
			break;
	}
}

///////////////////////////////////////////////////////////////////////////
//  Creates/deletes data buffers.
///////////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::SetupDataBuffers(int blocks)
{
	if(blocks>128)		//keep Block specifier bounded
		blocks = 128;
	if(blocks==0)
		blocks = 1;
	if( m_NumberOfBlocks != blocks )	//if blocks has changed
	{
		if(m_pDataBuffer)
		{
			for(int i=0; i<BLOCK_FIFO_SIZE; i++)
			{
				if(m_pDataBuffer[i])
				{
					delete[] m_pDataBuffer[i];
					m_pDataBuffer[i] = NULL;
				}
			}
			delete[] m_pDataBuffer;
		}
		m_NumberOfBlocks = blocks;
		// Allocate "main" array
		m_pDataBuffer = new short*[BLOCK_FIFO_SIZE];
		// Allocate "sub" arrays
		int sz;
		if( (DATA_BLOCK_SIZE*m_NumberOfBlocks) > m_FFTsize*2)
			sz = DATA_BLOCK_SIZE*m_NumberOfBlocks*2;
		else
			sz = m_FFTsize*4;
		for(int i=0; i<BLOCK_FIFO_SIZE; i++)
			m_pDataBuffer[i] = new short[sz];
//TRACE("sz=%d\n\r",sz);

	}
	m_FifoHead = 0;
	m_FifoTail = 0;
	m_FifoFFTTail = 0;
	m_BlockCount = 0;

}

///////////////////////////////////////////////////////////
//  Calculates the AD6620 I/Q Offset that is needed to
// eliminate a spur located at the NCO frequency
// This spur is a function of IF gain and the number of
// FIR taps in the AD6620.
///////////////////////////////////////////////////////////
double CSDR14XCtrl::CalcOffsetIQ()
{
double div;
	switch(m_IFgain & 0x1F)
	{
		case 0:
			div = 16.0;
			break;
		case 6:
			div = 8.0;
			break;
		case 12:
			div = 4.0;
			break;
		case 18:
			div = 2.0;
			break;
		case 24:
			div = 1.0;
			break;
	}
	return ((double)(m_RCFTaps)/(div*IQ_OFFSET_CONST));
}


///////////////////////////////////////////////////////////
//  Since the SDR-IQ processor does not have any EEROM,
// the reference calibration frequency is stored in the
// FTDI USB chips user EEROM area.  This routine reads it.
///////////////////////////////////////////////////////////
void CSDR14XCtrl::ReadSDRIQClockFrequency()
{
unsigned int val;
	if(SDRTYPE_IQ == m_SDRtype)
	{
		m_USBio.ReadIOMemory( IOEEROM_SAMPLERATE, &val);
		if( (val>50000000) && (val<70000000) )
		{
			m_ClockFrequency = val;
		}
		else	//write default to eerom
		{
			val = NOMINAL_ADFREQ;
			m_USBio.WriteIOMemory( IOEEROM_SAMPLERATE, val);
			m_ClockFrequency = val;
		}
	}
}

///////////////////////////////////////////////////////////
//  Since the SDR-IQ processor does not have any EEROM,
// the reference calibration frequency is stored in the
// FTDI USB chips user EEROM area.  This routine sends
// m_ClockFrequency back to the processor so it can be
// calibrated.
///////////////////////////////////////////////////////////
void CSDR14XCtrl::WriteSDRIQClockFrequency()
{
	if(SDRTYPE_IQ == m_SDRtype)
	{
		m_ASCPmsg.words.header = CI_RX_IN_SAMPLE_RATE_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_RX_IN_SAMPLE_RATE;
		m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_TestModeChannel;
		m_ASCPmsg.bytes.parms[1] = (unsigned __int8)(m_ClockFrequency&0xFF);
		m_ASCPmsg.bytes.parms[2] = (unsigned __int8)( (m_ClockFrequency>>8)&0xFF);
		m_ASCPmsg.bytes.parms[3] = (unsigned __int8)( (m_ClockFrequency>>16)&0xFF);
		m_ASCPmsg.bytes.parms[4] = (unsigned __int8)( (m_ClockFrequency>>24)&0xFF);
		m_USBio.SendASCPMsg( &m_ASCPmsg);
	}
}

//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
/////////// P r o p e r t y   A c c e s s    F u n c t i o n s //////
//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

short CSDR14XCtrl::GetBootRevision() 
{
	return m_BootRevision;
}

void CSDR14XCtrl::SetBootRevision(short nNewValue) 
{
	//Read Only
}

long CSDR14XCtrl::GetClockFrequency() 
{
	return m_ClockFrequency;
}

void CSDR14XCtrl::SetClockFrequency(long nNewValue) 
{
	if(SDRTYPE_IQ == m_SDRtype)
		m_USBio.WriteIOMemory( IOEEROM_SAMPLERATE, nNewValue);
	m_ClockFrequency = nNewValue;
	m_ASCPmsg.words.header = CI_RX_IN_SAMPLE_RATE_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_RX_IN_SAMPLE_RATE;
	m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_TestModeChannel;
	m_ASCPmsg.bytes.parms[1] = (unsigned __int8)(m_ClockFrequency&0xFF);
	m_ASCPmsg.bytes.parms[2] = (unsigned __int8)( (m_ClockFrequency>>8)&0xFF);
	m_ASCPmsg.bytes.parms[3] = (unsigned __int8)( (m_ClockFrequency>>16)&0xFF);
	m_ASCPmsg.bytes.parms[4] = (unsigned __int8)( (m_ClockFrequency>>24)&0xFF);
	m_USBio.SendASCPMsg( &m_ASCPmsg);
	SetModifiedFlag();
}

BSTR CSDR14XCtrl::GetDeviceName() 
{
	CString strResult;
	strResult = m_DeviceName;
	return strResult.AllocSysString();
}

void CSDR14XCtrl::SetDeviceName(LPCTSTR lpszNewValue) 
{
	//Read Only
}

short CSDR14XCtrl::GetFirmwareRevision() 
{
	return m_FirmwareRevision;
}

void CSDR14XCtrl::SetFirmwareRevision(short nNewValue) 
{
	//Read Only
}

short CSDR14XCtrl::GetHFgain() 
{
	return m_HFgain;
}

void CSDR14XCtrl::SetHFgain(short nNewValue) 
{
	m_HFgain = nNewValue;
	m_ASCPmsg.words.header = CI_RX_RF_GAIN_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_RX_RF_GAIN;
	m_ASCPmsg.bytes.parms[0] = (unsigned __int8)0;
	m_ASCPmsg.bytes.parms[1] = (unsigned __int8)m_HFgain;
	m_USBio.SendASCPMsg( &m_ASCPmsg);
	SetModifiedFlag();
}

short CSDR14XCtrl::GetIFfilter() 
{
	return m_IFfilter;
}

void CSDR14XCtrl::SetIFfilter(short nNewValue) 
{
C6620Params	filter;
	if(m_IFfilter != nNewValue)	//can't load new filter while running
		Stop();
	if(SDRTYPE_IQ == m_SDRtype)
	{	//0 thru 5 and 12 are the only valid values for the IQ
		if( nNewValue > 5 )
			nNewValue = 12;		//force it to 190KHz if out of range of SDR-IQ
		m_Complex = TRUE;
	}
	m_IFfilter = nNewValue;
	//calculate new sample rate based on new IF filter
	if(m_Complex)
	{
		filter.LoadFixedFilter(m_IFfilter+1);
		m_SampleRate = m_ClockFrequency/filter.m_TotalDecimationRate;
		m_RCFTaps = filter.m_RCFTaps;
		m_IQOffset = CalcOffsetIQ();
	}
	else
	{
		m_SampleRate = m_ClockFrequency;
	}
	m_FFT.SetFFTParams( m_Complex, m_FFTsize, m_FFTAve, m_FFTInvert, 0.0, m_IQOffset, m_SampleRate);
	SetModifiedFlag();
}

short CSDR14XCtrl::GetInputSource() 
{
	return m_InputSource;
}

void CSDR14XCtrl::SetInputSource(short nNewValue) 
{
	if( m_InputSource != nNewValue ) //must stop to change input src
		Stop();
	if(SDRTYPE_IQ == m_SDRtype)
	{
		nNewValue = 129;	//force SDR-IQ input to HF complex input source.
	}
	m_InputSource = nNewValue;
	if( m_InputSource&(1<<7) )
	{
		m_Complex = TRUE;
	}
	else
	{
		m_Complex = FALSE;
		m_SampleRate = m_ClockFrequency;
	}
	m_IQOffset = CalcOffsetIQ();
	m_FFT.SetFFTParams( m_Complex, m_FFTsize,m_FFTAve, m_FFTInvert, 0.0, m_IQOffset, m_SampleRate);
	SetModifiedFlag();
}

short CSDR14XCtrl::GetInterfaceVersion() 
{
	return m_InterfaceVersion;
}

void CSDR14XCtrl::SetInterfaceVersion(short nNewValue) 
{
	//Read Only
}

long CSDR14XCtrl::GetNCOfrequency() 
{
	return m_NCOfrequency;
}

void CSDR14XCtrl::SetNCOfrequency(long nNewValue) 
{
	m_NCOfrequency = nNewValue;
	m_ASCPmsg.words.header = CI_RX_FREQUENCY_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_RX_FREQUENCY;
	m_ASCPmsg.bytes.parms[0] = (unsigned __int8)0;
	m_ASCPmsg.bytes.parms[1] = (unsigned __int8)(m_NCOfrequency&0xFF);
	m_ASCPmsg.bytes.parms[2] = (unsigned __int8)( (m_NCOfrequency>>8)&0xFF);
	m_ASCPmsg.bytes.parms[3] = (unsigned __int8)( (m_NCOfrequency>>16)&0xFF);
	m_ASCPmsg.bytes.parms[4] = (unsigned __int8)( (m_NCOfrequency>>24)&0xFF);
	m_ASCPmsg.bytes.parms[5] = 1; //freq multiplier == x 1
	m_USBio.SendASCPMsg( &m_ASCPmsg);
	SetModifiedFlag();
}

BSTR CSDR14XCtrl::GetSerialNumber() 
{
	CString strResult;
	strResult = m_SerialNumber;
	return strResult.AllocSysString();
}

void CSDR14XCtrl::SetSerialNumber(LPCTSTR lpszNewValue) 
{
	//Read Only
}

short CSDR14XCtrl::GetIFgain() 
{
	return m_IFgain & 0x1F;
}

void CSDR14XCtrl::SetIFgain(short nNewValue) 
{
	m_IFgain = nNewValue;
	m_TestModeChannel = nNewValue & 0xE0;	//bits 7,6, and 5 are factory test mode bits
	m_IQOffset = CalcOffsetIQ();	//cacl a new IQ NCO spur removal offset
	m_ASCPmsg.words.header = CI_RX_IF_GAIN_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_RX_IF_GAIN;
	m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_TestModeChannel;
	m_ASCPmsg.bytes.parms[1] = (unsigned __int8)(m_IFgain & 0x1F);
	m_USBio.SendASCPMsg( &m_ASCPmsg);
	SetModifiedFlag();
}

long CSDR14XCtrl::GetSampleRate() 
{
	return m_SampleRate;
}

void CSDR14XCtrl::SetSampleRate(long nNewValue) 
{
	//Read Only
}

double CSDR14XCtrl::GetIQOffset() 
{
	return m_IQOffset;
}

void CSDR14XCtrl::SetIQOffset(double newValue) 
{
	//Read Only
}


//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=
//////////////////  E x p o s e d   M e t h o d s //////////////////////////
//+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=

//////////////////////////////////////////////////////////////////////
//  Called to Close the Aux I2C port. 
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::CloseI2CPort() 
{
	m_ASCPmsg.words.header = CI_CLOSE_I2C_PORT_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_CLOSE_I2C_PORT;
	m_ASCPmsg.bytes.parms[0] = 0;	//port number
}

//////////////////////////////////////////////////////////////////////
//  Called to Close the Aux SPI port. 
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::CloseSpiPort() 
{
	m_ASCPmsg.words.header = CI_CLOSE_SPI_PORT_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_CLOSE_SPI_PORT;
	m_ASCPmsg.bytes.parms[0] = 0;	//port number
}

//////////////////////////////////////////////////////////////////////
//  Called to Close the Aux Uart port. 
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::CloseUartPort() 
{
	m_ASCPmsg.words.header = CI_CLOSE_ASYNC_PORT_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_CLOSE_ASYNC_PORT;
	m_ASCPmsg.bytes.parms[0] = 0;	//port number
	m_USBio.SendASCPMsg( &m_ASCPmsg);
}

//////////////////////////////////////////////////////////////////////
//  Called to get doubles of SDR-14 captured data
//////////////////////////////////////////////////////////////////////
long CSDR14XCtrl::GetDoubleData(double FAR* DataArray) 
{
long length;
	EnterCriticalSection(&m_CriticalSection);
	if(m_pDataBuffer)
	{
		length = m_NumberOfBlocks*DATA_BLOCK_SIZE;
		if(m_FifoHead != m_FifoTail)
		{
			for(int i=0; i<length; i++)
				DataArray[i] = (double)m_pDataBuffer[m_FifoTail][i] + m_IQOffset;
			m_FifoTail++;
			m_FifoTail &= (BLOCK_FIFO_SIZE-1);
		}
		else
		{
			for(int i=0; i<length; i++)
				DataArray[i] = 0.0;
		}
	}
	else
		length = 0;
	m_WindowsMsgCount = 0;
	LeaveCriticalSection(&m_CriticalSection);
	if(m_Complex)	//return actual number of data samples
		return length/2;
	else
		return length;
}

//////////////////////////////////////////////////////////////////////
//  Method to calculate and obtain the FFT Data
//  The user buffer 'DataArray' MUST be large enough to hold all
//  the FFT data points
//////////////////////////////////////////////////////////////////////
BOOL CSDR14XCtrl::GetFftDoubleData(double FAR* DataArray) 
{
int i = 0;
BOOL ovr = FALSE;
int InSize;
	if(m_Complex)
		InSize = m_FFTsize*2;
	else
		InSize = m_FFTsize;

	if(m_pDataBuffer)
	{
		while( (InSize*i) < (m_NumberOfBlocks*DATA_BLOCK_SIZE) )
		{
			m_FFT.PutInFFT( (m_pDataBuffer[m_FifoFFTTail] + i*InSize ));
			ovr |= m_FFT.GetDoubleFFTData(0, InSize/2, DataArray);
			i++;
		}
		m_FifoFFTTail++;
		m_FifoFFTTail &= (BLOCK_FIFO_SIZE-1);
	}
	m_WindowsMsgCount = 0;
	ovr = ovr || m_ADOverload;
	m_ADOverload = FALSE;
	return ovr;
}

//////////////////////////////////////////////////////////////////////
//The FFT is calculated and data translated to be useful in ploting.
// The range of Start to End frequencies are mapped to the users
// plot screen size so the users buffer will be filled with an array
// of integers whos value is the y pixel height and the index of the
//  array is the x pixel coordinate.
// The function returns TRUE if the input is overloaded
//   This routine converts the data to 32 bit integers and is useful
//   when displaying fft data on the screen. 
//		PlotHeght = Plot height in pixels of the user plot screen
//		PlotWidth =  Plot width in pixels of the user plot screen
//
//		StartFreq = freq in Hz
//		EndFreq = freq in Hz
//         For Real FFT, the range of frequencies are 0 to 33,333,333Hz
//         For Complex FFT, the range of frequencies are +/- (m_SampleRate/2)
//			Values are clamped so the user can specify -33,333,333 for Start
//	        and +33,333,333 for End and obtain the full range for any FFT.
//
//		MaxdB = FFT dB level corresponding to output value == 0
//			must be <= to K_MAXDB (0)
//		MindB = FFT dB level  corresponding to output value == MaxHeight
//			must be >= to K_MINDB (-140)
//////////////////////////////////////////////////////////////////////
BOOL CSDR14XCtrl::GetFftScreenData(long PlotHeght, long PlotWidth, double MaxdB, double MindB, long StartFreq, long EndFreq, long FAR* PlotBuf) 
{
int i = 0;
BOOL ovr = FALSE;
int InSize;
	if(m_Complex)
		InSize = m_FFTsize*2;
	else
		InSize = m_FFTsize;
	if(m_pDataBuffer)
	{
		while( (InSize*i) < (m_NumberOfBlocks*DATA_BLOCK_SIZE) )
		{
			m_FFT.PutInFFT( (m_pDataBuffer[m_FifoFFTTail]  + i*InSize ) );
			ovr |= m_FFT.GetScreenIntegerFFTData(PlotHeght, PlotWidth, MaxdB, MindB, StartFreq, EndFreq, PlotBuf);
			i++;
		}
		m_FifoFFTTail++;
		m_FifoFFTTail &= (BLOCK_FIFO_SIZE-1);
	}
	m_WindowsMsgCount = 0;
	ovr = ovr || m_ADOverload;
	m_ADOverload = FALSE;
	return ovr;
}

//////////////////////////////////////////////////////////////////////
//  Called to get 16 bit Integers(shorts) of SDR-14 captured data
//////////////////////////////////////////////////////////////////////
long CSDR14XCtrl::GetIntegerData(short FAR* DataArray) 
{
long length;
	EnterCriticalSection(&m_CriticalSection);
	if(m_pDataBuffer)
	{
		length = m_NumberOfBlocks*DATA_BLOCK_SIZE;
		if(m_FifoHead != m_FifoTail)
		{
			for(int i=0; i<length; i++)
				DataArray[i] = m_pDataBuffer[m_FifoTail][i];
			m_FifoTail++;
			m_FifoTail &= (BLOCK_FIFO_SIZE-1);
		}
		else
		{
//TRACE("?");
			for(int i=0; i<length; i++)
				DataArray[i] = 0;
		}
	}
	else
		length = 0;
	m_WindowsMsgCount = 0;
	LeaveCriticalSection(&m_CriticalSection);
//TRACE(":");
	if(m_Complex)	//return actual number of data samples
		return length/2;
	else
		return length;
}

//////////////////////////////////////////////////////////////////////
//  Called to get 32 bit longs of SDR-14 captured data
//////////////////////////////////////////////////////////////////////
long CSDR14XCtrl::GetLongData(long FAR* DataArray) 
{
long length;
	EnterCriticalSection(&m_CriticalSection);
	if(m_pDataBuffer)
	{
		length = m_NumberOfBlocks*DATA_BLOCK_SIZE;
		if(m_FifoHead != m_FifoTail)
		{
			for(int i=0; i<length; i++)
				DataArray[i] = (long)m_pDataBuffer[m_FifoTail][i];
			m_FifoTail++;
			m_FifoTail &= (BLOCK_FIFO_SIZE-1);
		}
		else
		{
			for(int i=0; i<length; i++)
				DataArray[i] = 0;
		}
	}
	else
		length = 0;
	m_WindowsMsgCount = 0;
	LeaveCriticalSection(&m_CriticalSection);
	if(m_Complex)	//return actual number of data samples
		return length/2;
	else
		return length;
}

//////////////////////////////////////////////////////////////////////
//  Called to get Aux port Uart data. Called after receiving a 
//  UartDataRdy Event
//////////////////////////////////////////////////////////////////////
short CSDR14XCtrl::GetUartData(short FAR* UartRxArray) 
{
short length = 0;
	EnterCriticalSection(&m_CriticalSection);
	while(m_AuxRxHead != m_AuxRxTail)
	{
		UartRxArray[length++] = (unsigned short)m_AuxRxQue[m_AuxRxTail++];
		m_AuxRxTail &= AUX_QUE_MODMASK;	//deal with queue ptr wrap around
	}
	LeaveCriticalSection(&m_CriticalSection);
//TRACE("\n\rl=%d", length);
	return length;
}

//////////////////////////////////////////////////////////////////////
//  Called to Initialize the Control and start the worker thread that
//  runs the USB port.
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::InitControl() 
{
	if(!m_ThreadRunning)	//no need to restart thread if running
	{
#ifdef OLEMISC_INVISIBLEATRUNTIME_FIX
		//(dec) 9-Oct-04 Added to fix OLEMISC_INVISIBLEATRUNTIME problem with VB
		m_bDrawCtl = FALSE;
		SetControlSize(0, 0);
#endif
		EndThreads();	//if thread is already running stop it
		//Start up USB worker thread
		m_pUSBThread = AfxBeginThread(CSDR14XCtrl::USBThreadProc,
									this,
									THREAD_PRIORITY_TIME_CRITICAL,
//									THREAD_PRIORITY_ABOVE_NORMAL,
									0,
									CREATE_SUSPENDED);
		m_pUSBThread->m_bAutoDelete = FALSE;
		m_ThreadRunning = TRUE;
		m_pUSBThread->ResumeThread();
		//Start up FIFO worker thread
		m_pFIFOThread = AfxBeginThread(CSDR14XCtrl::FIFOThreadProc,
									this,
//									THREAD_PRIORITY_TIME_CRITICAL,
									THREAD_PRIORITY_ABOVE_NORMAL,
									0,
									CREATE_SUSPENDED);
		m_pFIFOThread->m_bAutoDelete = FALSE;
		m_pFIFOThread->ResumeThread();
	}
}


//////////////////////////////////////////////////////////////////////
//  Called to Open the Aux I2C port. Mastermode only write only. 
//  AdrWidth:
//		0== 7 bit address mode
//		1== 10 bit address mode
//		255== factory test (Stops USB thread)
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::OpenI2CPort(short AdrWidth) 
{
	if( 255 == AdrWidth)
	{
		EndThreads();
	}
	else
	{
		m_ASCPmsg.words.header = CI_OPEN_I2C_PORT_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_OPEN_I2C_PORT;
		m_ASCPmsg.bytes.parms[0] = 0;	//port number
		m_ASCPmsg.bytes.parms[1] = 3;	//DataItem used for transfer
		m_ASCPmsg.bytes.parms[2] = 1;	//Master mode(master only supported)
		m_ASCPmsg.bytes.parms[3] = 0;	//slave address Low not used
		m_ASCPmsg.bytes.parms[4] = 0;	//slave address High not used
		m_ASCPmsg.bytes.parms[5] = (unsigned __int8)AdrWidth;//adr width(7 ot 10)
		m_ASCPmsg.bytes.parms[6] = 0;	//speed not used
		m_USBio.SendASCPMsg( &m_ASCPmsg);
	}
}

//////////////////////////////////////////////////////////////////////
//  Called to Open the Aux SPI port. Mastermode only write only. 
//  ClkPolarity:
//		0== clock Idle state=Low
//		1== clock Idle state=High
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::OpenSpiPort(short ClkPolarity) 
{
	m_SPIClockPol = ClkPolarity;
	m_ASCPmsg.words.header = CI_OPEN_SPI_PORT_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_OPEN_SPI_PORT;
	m_ASCPmsg.bytes.parms[0] = 0;	//port number
	m_ASCPmsg.bytes.parms[1] = 3;	//DataItem used for transfer
	m_ASCPmsg.bytes.parms[2] = 1;	//Master mode(master only supported)
	m_ASCPmsg.bytes.parms[3] = (unsigned __int8)m_SPIClockPol;//clk setting
	m_ASCPmsg.bytes.parms[4] = (unsigned __int8)m_NumberOfSPIBits;	//Number bits in transfer
	m_ASCPmsg.bytes.parms[5] = 0;	//bit rate not used
	m_ASCPmsg.bytes.parms[6] = 0;	//bit rate not used
	m_ASCPmsg.bytes.parms[7] = 0;	//bit rate not used
	m_ASCPmsg.bytes.parms[8] = 0;	//bit rate not used
	m_USBio.SendASCPMsg( &m_ASCPmsg);
}

//////////////////////////////////////////////////////////////////////
//  Called to Open the Aux Uart port. 
//  BaudRate range is: 1200 to 19200 bits/sec
//  Parity: 0==no parity, 1== Odd parity, 2==Even parity
//	FlowControl: TRUE == Uses RTS, FALSE==No Flow control
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::OpenUartPort(long BaudRate, short Parity, BOOL FlowControl) 
{
	m_AuxRxHead = 0;
	m_AuxRxTail = 0;
	m_ASCPmsg.words.header = CI_OPEN_ASYNC_PORT_SETRESPLEN;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
	m_ASCPmsg.words.CItem = CI_OPEN_ASYNC_PORT;
	m_ASCPmsg.bytes.parms[0] = 0;	//port number
	m_ASCPmsg.bytes.parms[1] = 2;	//DataItem used for transfer
	m_ASCPmsg.bytes.parms[2] = 8;	//bits in data(always 8)
	m_ASCPmsg.bytes.parms[3] = (unsigned __int8)Parity;	//parity setting
	m_ASCPmsg.bytes.parms[4] = 2;	//stop bits(always 2)
	m_ASCPmsg.bytes.parms[5] = (unsigned __int8)FlowControl;//flow control
	m_ASCPmsg.bytes.parms[6] = (unsigned __int8)(BaudRate&0xFF);
	m_ASCPmsg.bytes.parms[7] = (unsigned __int8)( (BaudRate>>8)&0xFF);
	m_ASCPmsg.bytes.parms[8] = (unsigned __int8)( (BaudRate>>16)&0xFF);
	m_ASCPmsg.bytes.parms[9] = (unsigned __int8)( (BaudRate>>24)&0xFF);
	m_USBio.SendASCPMsg( &m_ASCPmsg);

}

//////////////////////////////////////////////////////////////////////
//  Method to reset the FFT averaging buffers
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::ResetFftAverage() 
{
	m_FFT.ResetFFT();
}

//////////////////////////////////////////////////////////////////////
//  Method to send a raw data message to the AD6620
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::Send6620RawCmd(short Address, short D0, short D1, short D2, short D3, short D4) 
{
	m_ASCPmsg.words.header = 2+7;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_DATA_ITEM1;
	m_ASCPmsg.dibytes.parms[0] = (unsigned __int8)(Address)&0x00FF;
	m_ASCPmsg.dibytes.parms[1] = (unsigned __int8)(Address>>8)&0x00FF;
	m_ASCPmsg.dibytes.parms[2] = (unsigned __int8)D0&0x00FF;
	m_ASCPmsg.dibytes.parms[3] = (unsigned __int8)D1&0x00FF;
	m_ASCPmsg.dibytes.parms[4] = (unsigned __int8)D2&0x00FF;
	m_ASCPmsg.dibytes.parms[5] = (unsigned __int8)D3&0x00FF;
	m_ASCPmsg.dibytes.parms[6] = (unsigned __int8)D4&0x00FF;
	m_USBio.SendASCPMsg( &m_ASCPmsg);
}

//////////////////////////////////////////////////////////////////////
//  Called to send 'NumDataItems' bytes in pData to the Aux I2C port.
//  long data is truncated to 8 bit data.
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::SendI2CData(short FAR* TxDataArray, short NumDataItems) 
{
	if(NumDataItems>MAX_AUX_BYTES)
		NumDataItems = MAX_AUX_BYTES;
	for(int i=0; i<NumDataItems; i++)
		m_ASCPmsg.bytes.parms[i] = (unsigned __int8)TxDataArray[i];
	m_ASCPmsg.words.header = NumDataItems;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_DATA_ITEM3;
	m_USBio.SendASCPMsg( &m_ASCPmsg);
}


//////////////////////////////////////////////////////////////////////
//  Called to send Data using the Aux SPI port.
//	"NumberBits" specifies number of bits to send(1 to 64)
//  "DataHigh32" long 32 bit 'Data' contains bits 32 to 63 of data.
//  "DataLow32" long 32 bit 'Data' contains bits 0 to 31 of data.
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::SendSpiData(long Data32High, long Data32Low, short NumberBits) 
{
	if( NumberBits != m_NumberOfSPIBits)
	{	//if changed number of bits, reopen port with new value
		m_NumberOfSPIBits = NumberBits;
		OpenSpiPort(m_SPIClockPol);
	}
	m_ASCPmsg.words.header = 2+8;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_DATA_ITEM3;
	m_ASCPmsg.dibytes.parms[0] = (unsigned __int8)(Data32Low)&0x000000FF;
	m_ASCPmsg.dibytes.parms[1] = (unsigned __int8)(Data32Low>>8)&0x000000FF;
	m_ASCPmsg.dibytes.parms[2] = (unsigned __int8)(Data32Low>>16)&0x000000FF;
	m_ASCPmsg.dibytes.parms[3] = (unsigned __int8)(Data32Low>>24)&0x000000FF;
	m_ASCPmsg.dibytes.parms[4] = (unsigned __int8)(Data32High)&0x000000FF;
	m_ASCPmsg.dibytes.parms[5] = (unsigned __int8)(Data32High>>8)&0x000000FF;
	m_ASCPmsg.dibytes.parms[6] = (unsigned __int8)(Data32High>>16)&0x000000FF;
	m_ASCPmsg.dibytes.parms[7] = (unsigned __int8)(Data32High>>24)&0x000000FF;
	m_USBio.SendASCPMsg( &m_ASCPmsg);
}

//////////////////////////////////////////////////////////////////////
//  Called to send 'NumDataItems' bytes in pData to the Aux Uart port.
//  long data is truncated to 8 bit data.
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::SendUartData(short FAR* UartTxArray, short NumDataItems) 
{
	m_ASCPmsg.words.header = NumDataItems+2;
	m_ASCPmsg.bytes.b1 |= TYPE_HOST_DATA_ITEM2;
	if(NumDataItems>MAX_AUX_BYTES)
		NumDataItems = MAX_AUX_BYTES;
	for(int i=0; i<NumDataItems; i++)
		m_ASCPmsg.dibytes.parms[i] = (unsigned __int8)UartTxArray[i];
	m_USBio.SendASCPMsg( &m_ASCPmsg);
}

//////////////////////////////////////////////////////////////////////
//  Method called to start and setup the FFT parameters
//   FftPoints = number of FFT points to calculate.
//   Fftaverages = number of running bin by bin averages to calculate.
//   FftInverted = TRUE flips spectrum for cases of inverted IF's or downconvertors
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::SetupFFT(long FftPoints, short Fftaverages, BOOL FftInverted) 
{
	m_FFTsize = FftPoints;		//save local copies of parameters
	m_FFTInvert = FftInverted;
	m_FFTAve = Fftaverages;
	m_FFT.SetFFTParams( m_Complex, m_FFTsize,m_FFTAve, m_FFTInvert, 0.0, m_IQOffset, m_SampleRate);
}

//////////////////////////////////////////////////////////////////////
//  Method called to start SDR-14 capturing data
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::Start(short Blocks, short Mode) 
{
	if(!m_USBio.m_Running && m_USBio.IsPortOpen() )	//ignor if already running or usb port closed
	{
		m_CaptureMode = Mode;
		if(m_InputSource&0x80) //if using AD6620 downconverter
		{
			m_SampleRate = m_ClockFrequency/m_USBio.Load6620Params(m_IFfilter);
		}
		else	//is raw A/D sampling mode
		{
			m_SampleRate = m_ClockFrequency;
			m_USBio.m_Last6620FilterLoaded = -1;
			SetModifiedFlag(TRUE);	//make sure NCO reloads next time used
		}
		if( IsModified() )	//if properties changed reload the world
		{
			SetIFgain(m_IFgain);
			SetHFgain(m_HFgain);
			SetClockFrequency(m_ClockFrequency); 
			SetNCOfrequency(m_NCOfrequency); 
			SetModifiedFlag(FALSE);
		}
		if( (m_CaptureMode == CAPTURE_MODE_CONTIGUOUS) &&
					(m_SampleRate>MAX_CONTIGUOUS_FREQ) )
		{	//can't run contiguous so make continuous
			m_CaptureMode = CAPTURE_MODE_CONTINUOUS;
		}
		// calcualte SDR-14 data watchdog timer value for data0 Acking
		if( 0x03 == m_CaptureMode)	//if continuous HW sync mode
			m_DataAckCounterLoadVal = 1;
		else if(m_SampleRate<=MAX_CONTIGUOUS_FREQ)
			m_DataAckCounterLoadVal = m_SampleRate/2048;
		else
			m_DataAckCounterLoadVal = 60;
		m_DataAckCounter = m_DataAckCounterLoadVal;
		m_ADOverload = FALSE;
		m_IQOffset = CalcOffsetIQ();	//make sure IQOffset is up to date
		m_FFT.SetFFTParams( m_Complex, m_FFTsize,m_FFTAve, m_FFTInvert, 0.0, m_IQOffset, m_SampleRate);
		SetupDataBuffers(Blocks);	//create data fifo buffers
		//send start command to SDR-14
		m_ASCPmsg.words.header = CI_RX_STATE_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_RX_STATE;
		m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_InputSource;
		m_ASCPmsg.bytes.parms[1] = RX_STATE_ON;
		m_ASCPmsg.bytes.parms[2] = (unsigned __int8)m_CaptureMode;
		m_ASCPmsg.bytes.parms[3] = (unsigned __int8)Blocks;
		m_USBio.SendASCPMsg( &m_ASCPmsg);
	}
}

//////////////////////////////////////////////////////////////////////
//  Method called to stop the SDR-14 data capturing
//////////////////////////////////////////////////////////////////////
void CSDR14XCtrl::Stop() 
{
	if(m_USBio.m_Running && m_USBio.IsPortOpen() )	//ignor if alreay stopped
	{
		m_ASCPmsg.words.header = CI_RX_STATE_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_RX_STATE;
		m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_InputSource;	//active rx channel
		m_ASCPmsg.bytes.parms[1] = RX_STATE_IDLE;
		m_ASCPmsg.bytes.parms[2] = 0;
		m_ASCPmsg.bytes.parms[3] = 0;
		m_USBio.SendASCPMsg( &m_ASCPmsg);
	}
}


void CSDR14XCtrl::ErrorStop()
{
	if(m_USBio.m_Running && m_USBio.IsPortOpen() )	//ignor if alreay stopped
	{
		m_ASCPmsg.words.header = CI_RX_STATE_SETRESPLEN;
		m_ASCPmsg.bytes.b1 |= TYPE_HOST_SET_CITEM;
		m_ASCPmsg.words.CItem = CI_RX_STATE;
		m_ASCPmsg.bytes.parms[0] = (unsigned __int8)m_InputSource;	//active rx channel
		m_ASCPmsg.bytes.parms[1] = RX_STATE_IDLE;
		m_ASCPmsg.bytes.parms[2] = 0;
		m_ASCPmsg.bytes.parms[3] = 0;
		m_USBio.SendASCPMsg( &m_ASCPmsg);
	}
}

