// USBio.h: interface for the CUSBio class.
//  A class for communicating with the SDR-14 over a USB port
//////////////////////////////////////////////////////////////////////
// Copyright 2003.    Moe Wheatley AE4JY  <http://www.moetronix.com>
//
//This program is free software; you can redistribute it and/or
//modify it how ever you wish.  Its primary goal is to provide an
// example for 3rd party software developers.  Support will be limited
// depending on my available time.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////

#if !defined(AFX_USBIO_H__170C0E46_F7C2_4E2C_9B7F_54A4DDBA4AE3__INCLUDED_)
#define AFX_USBIO_H__170C0E46_F7C2_4E2C_9B7F_54A4DDBA4AE3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "FTD2XX.h"
#include "6620params.h"
#include "AscpDefs.h"	//local copy of ASCP defs

#define EEROM_SIZE 32

//////////////////////////////////////////////////////////////////////////////
// highly non-portable ASCP message buffer structure but....its fast and easy
//////////////////////////////////////////////////////////////////////////////
typedef union
{
	struct bts
	{
		unsigned __int8 b0;
		unsigned __int8 b1;
		unsigned __int8 b2;
		unsigned __int8 b3;
		unsigned __int8 parms[MAX_MSG_LENGTH-4];
	}bytes;
	struct dbts
	{
		unsigned __int8 b0;
		unsigned __int8 b1;
		unsigned __int8 parms[MAX_MSG_LENGTH-2];
	}dibytes;
	struct dwrds
	{
		unsigned __int16 header;
		unsigned __int16 parms[(MAX_MSG_LENGTH-2)/2];
	}diwords;
	struct wrds
	{
		unsigned __int16 header;
		unsigned __int16 CItem;
		unsigned __int16 parms[(MAX_MSG_LENGTH-4)/2];
	}words;
	struct dlngs
	{
		unsigned __int16 header;
		unsigned __int16 parms[(MAX_MSG_LENGTH-2)/2];
	}data;
	struct lngs
	{
		unsigned __int32 header;
		unsigned __int32 parms[(MAX_MSG_LENGTH-4)/4];
	}longs;
	unsigned __int8 buf[MAX_MSG_LENGTH];
}ASCPstruct;


class CUSBio  
{
public:
	BOOL IsPortOpen(){ return m_USBPortOpen;};
	CUSBio();
	virtual ~CUSBio();
	// Externally exposed functions
	int Load6620Params( int filter );
	void USBthreadProc(  LPVOID pCtrl );
	void FIFOThreadProc(LPVOID pCtrl);
	void SendASCPMsg( ASCPstruct* pASCPmsg);
	void ReadIOMemory( int adr, unsigned int* pData);
	void WriteIOMemory( int adr, unsigned int Data);
	// Externally exposed variables
	BOOL m_Running;
	int m_Last6620FilterLoaded;


private:
	// local functions
	void Send6620Msg( int adr, unsigned int data);
	void CloseUSBPort();
	void DeleteResources();
	BOOL OpenUSBdll();
	void SetSDRIQSampleRateFromEEROM(unsigned int rate);

	// local variables
	BOOL m_USBPortOpen;
	int m_RxMsgLength;
	int m_RxMsgIndex;
	int m_MsgState;
	int m_FifoHead;
	int m_FifoTail;
	unsigned char m_EEROMData[EEROM_SIZE];
	unsigned __int8* m_pUSBBuffer;
	unsigned __int8* m_pUSBFifo;
	FT_STATUS m_FTStatus;
	FT_HANDLE m_FTHandle;
	HMODULE m_hmodule;
	HANDLE m_hReadEvent;
	HANDLE m_hFifoWriteEvent;
	CWinThread *m_pUSBFifoThread;
	FT_PROGRAM_DATA m_FT_ProgramData;
	CRITICAL_SECTION m_CriticalSection;	//for keeping threads from stomping on each other
	ASCPstruct m_rxASCPmsg;
	ASCPstruct m_txASCPmsg;
	C6620Params m_6620params;
};

#endif // !defined(AFX_USBIO_H__170C0E46_F7C2_4E2C_9B7F_54A4DDBA4AE3__INCLUDED_)
